"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const ts_simple_ast_1 = require("ts-simple-ast");
/**
 
# Description

see https://github.com/Microsoft/TypeScript/issues/23552

# Attacks

This refactor doesn't attack any diagnostic error

# Example

```
function foo (a: number, b: string[], c: (n:number)=>boolean[], d?: boolean){}
```

select a fragment of the parameter list will suggest "declare interface from named parameters" and if applied will change the code to :

```
interface FooOptions{
  a: number
  b: string[]
  c?: boolean
}
function foo({a,b,c}: FooOptions){}
```
*/
exports.toNamedParameters = {
    name: 'toNamedParameters',
    config: {},
    predicate: (arg) => {
        const param = typescript_ast_util_1.findAscendant(arg.containingTargetLight, ts.isParameter, true);
        if (!param) {
            arg.log('toNamedParameters predicate false because no  Parameter ascendant found - containingTarget was ' + typescript_ast_util_1.getKindName(arg.containingTargetLight));
            return false;
        }
        return true;
    },
    description: (arg) => `Convert to Named Parameters`,
    apply: (arg) => {
        const functionLikeDeclaration = arg.simpleNode.getAncestors().find(ts_simple_ast_1.TypeGuards.isFunctionLikeDeclaration);
        if (!functionLikeDeclaration) {
            arg.log('toNamedParameters not applied because FunctionLikeDeclaration ancestor not found');
            return;
        }
        const container = functionLikeDeclaration.getParent();
        if (!ts_simple_ast_1.TypeGuards.isStatementedNode(container)) {
            arg.log('FunctionLikeDeclaration.getParent not applied because is not TypeGuards.isStatementedNode(container)');
            return;
        }
        let interfaceName = functionLikeDeclaration.getName() ? functionLikeDeclaration.getName() + '' : 'UnnamedOptions';
        interfaceName = interfaceName.substring(0, 1).toUpperCase() + interfaceName.substring(1, interfaceName.length);
        const interfaceStructure = {
            name: interfaceName,
            properties: functionLikeDeclaration.getParameters().filter(param => !param.getTypeNode() || param.getTypeNode().getKind() !== ts.SyntaxKind.FunctionType) // TODO: this will ignore/wrong-print parameters of type function that have an assignament
                .map(property => ({
                name: property.getName(),
                type: property.getTypeNode() && property.getTypeNode().getText() || property.getType() && property.getType().getText(),
                hasQuestionToken: property.hasQuestionToken()
            })),
            methods: functionLikeDeclaration.getParameters().filter(param => param.getTypeNode() && param.getTypeNode().getKind() == ts.SyntaxKind.FunctionType).map(method => ({
                name: method.getName(),
                returnType: method.getTypeNode() && method.getTypeNode().getReturnTypeNode() && method.getTypeNode().getReturnTypeNode().getText(),
                parameters: method.getTypeNode().getParameters().map(param => ({
                    name: param.getName(),
                    type: param.getTypeNode() && param.getTypeNode().getText() || param.getType() && param.getType().getText(),
                    hasQuestionToken: param.hasQuestionToken()
                })),
                hasQuestionToken: method.hasQuestionToken()
            })),
        };
        container.insertInterface(functionLikeDeclaration.getChildIndex(), interfaceStructure);
        const parameterDeclarationStructure = {
            name: '{' + functionLikeDeclaration.getParameters().map(param => param.getName() + (param.getInitializer() ? (' = ' + param.getInitializer().getText()) : '')).join(', ') + '}',
            type: interfaceName
        };
        functionLikeDeclaration.getParameters().forEach(param => param.remove());
        functionLikeDeclaration.addParameter(parameterDeclarationStructure);
    }
};
//# sourceMappingURL=toNamedParams.js.map